"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.posthtmlHash = exports.hash = exports.replaceHash = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var hasha_1 = __importDefault(require("hasha"));
var DEFAULT_HASH_LENGTH = 20;
var DEFAULT_PATTERN = new RegExp(/\[hash.*]/g);
function replaceHash(str, buffer, exp, hashLength) {
    var match = str.match(exp);
    var _a = match[0].replace(/\[|]/g, "").split(":"), _ = _a[0], len = _a[1];
    return str.replace(exp, hasha_1["default"](buffer).slice(0, Number(len) || hashLength));
}
exports.replaceHash = replaceHash;
function plugin(options) {
    var hashedUrls = new Map();
    return function posthtmlHash(tree) {
        var exp = (options === null || options === void 0 ? void 0 : options.pattern) || DEFAULT_PATTERN;
        var hashLength = (options === null || options === void 0 ? void 0 : options.hashLength) || DEFAULT_HASH_LENGTH;
        var transformPath = (options === null || options === void 0 ? void 0 : options.transformPath) || (function (filepath) { return filepath; });
        tree.match([{ attrs: { href: exp } }, { attrs: { src: exp } }, { attrs: { content: exp } }], function (node) {
            var _node = node;
            var _a = _node.attrs, href = _a.href, src = _a.src, content = _a.content;
            var fileName = href || src || content;
            var transformedFileName = transformPath(fileName);
            var pathToFile = (options === null || options === void 0 ? void 0 : options.path) || "";
            var file = path_1["default"].join(process.cwd(), pathToFile, transformedFileName);
            if (hashedUrls.has(file)) {
                var existingHashedUrl = hashedUrls.get(file);
                if (href)
                    _node.attrs.href = existingHashedUrl;
                if (src)
                    _node.attrs.src = existingHashedUrl;
                if (content)
                    _node.attrs.content = existingHashedUrl;
            }
            else {
                if (fs_1["default"].existsSync(file)) {
                    var buffer = fs_1["default"].readFileSync(file);
                    var hashedFileName = replaceHash(fileName, buffer, exp, hashLength);
                    var transformedHashedFileName = transformPath(hashedFileName);
                    var hashedFile = path_1["default"].join(process.cwd(), pathToFile, transformedHashedFileName);
                    fs_1["default"].renameSync(file, hashedFile);
                    hashedUrls.set(file, hashedFileName);
                    if (href)
                        _node.attrs.href = hashedFileName;
                    if (src)
                        _node.attrs.src = hashedFileName;
                    if (content)
                        _node.attrs.content = hashedFileName;
                }
                else {
                    console.log("File does not exist:", file);
                }
            }
            return node;
        });
    };
}
exports.hash = plugin;
exports.posthtmlHash = plugin;
exports["default"] = plugin;
